/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/


#include "udm_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <errno.h>

#include "udm_common.h"


typedef struct
{
  UDM_VALUE generic;
  int Value;
} UDM_VALUE_INT;


typedef struct
{
  UDM_VAR_HEADER Header;
  UDM_VALUE_INT Value;
} UDM_VAR_INT;


static udm_rc_t
UdmValueIntDump(const char *Data, const char *name, FILE *f)
{
  return UDM_NOTARGET;
}


static udm_var_flag_t
UdmValueIntFlags(const char *Data)
{
  return UDM_VARFLAG_NONE;
}


static udm_secno_t
UdmValueIntSecno(const char *Data)
{
  return 0;
}


static size_t
UdmValueIntMemUsed(const char *Data)
{
  return sizeof(int);
}


static void
UdmValueIntDestructor(char *Data)
{
}


static udm_rc_t
UdmValueIntCopy(char *D, const char *S)
{
  UDM_ASSERT(0); /* TODO34: Add CloneVar(), with reference */
  UDM_BZERO(D, sizeof(int));
  return UDM_OK;
}


static udm_rc_t
UdmValueIntConstructor(char *Data,
                       const UDM_VALUE **args, size_t nargs)
{
  ((int *) Data)[0]= 0;
  return UDM_OK;
}


static size_t
UdmValueIntVarSize(void)
{
  return sizeof(UDM_VAR_INT);
}


static size_t
UdmValueIntValueSize(void)
{
  return sizeof(UDM_VALUE_INT);
}


static size_t
UdmValueIntDataOffset(void)
{
  return offsetof(UDM_VALUE_INT, Value);
}


static size_t
UdmValueIntDataSize(void)
{
  return sizeof(int);
}


static size_t
UdmValueIntDataAlignment(void)
{
  return sizeof(int);
}


static void
UdmValueIntGetConstStr(const char *Data, UDM_CONST_STR *Val)
{
  static const UDM_CONST_STR name= {UDM_CSTR_WITH_LEN("#Int#")};
  *Val= name;
}


static void
UdmValueIntGetBool(const char *Data, udm_bool_t *val)
{
  *val= ((const int *) Data)[0];
}


static void
UdmValueIntGetInt(const char *Data, int *val)
{
  *val= ((const int *) Data)[0];
}


static void
UdmValueIntGetUInt(const char *Data, unsigned int *val)
{
  *val= ((const int *) Data)[0];
}


static void
UdmValueIntGetDouble(const char *Data, double *val)
{
  *val= ((const int *) Data)[0];
}


static udm_rc_t
UdmValueIntSetInt(char *Data, int value)
{
  ((int *) Data)[0]= value;
  return UDM_OK;
}


static udm_rc_t
UdmValueIntSetDouble(char *Data, double value)
{
  ((int *) Data)[0]= value;
  return UDM_OK;
}


static udm_rc_t
UdmValueIntConvertCharset(char *Data, UDM_CONV *conv, int flags)
{
  return UDM_NOTARGET;
}


static udm_rc_t
UdmValueIntAppendStrn(char *Data, const char *str, size_t length)
{
  return UDM_NOTARGET;
}


static udm_rc_t
UdmValueIntSetStrn(char *Data, const char *str, size_t length)
{
  char tmp[128];
  udm_snprintf(tmp, sizeof(tmp), "%.*s", (int) length, str);
  ((int *) Data)[0]= atoi(tmp);
  return UDM_OK;
}


static udm_rc_t
UdmValueIntSetConv(char *Data, UDM_CONV *conv, int cnvflags,
                      const char *src, size_t srclen)
{
  return UDM_NOTARGET;
}


static udm_rc_t
UdmValueIntAppendConv(char *Data, size_t maxlen,
                      UDM_CONV *cnv, int cnvflags,
                      const char *src, size_t srclen)
{
  return UDM_NOTARGET;
}


static udm_rc_t
UdmValueIntPrintToFile(const char *Data, FILE *f)
{
  fprintf(f, "%d", ((const int *)Data)[0]);
  return UDM_OK;
}


static udm_rc_t
UdmValueIntPrintToDSTR(const char *Data, UDM_DSTR *dstr)
{
  char tmp[64];
  size_t length= udm_snprintf(tmp, sizeof(tmp), "%d", ((const int *)Data)[0]);
  return UdmDSTRAppend(dstr, tmp, length);
}


UDM_VALUE_HANDLER UdmValueHandlerInt=
{
  UDM_VALUE_HANDLER_TYPE_INT,
  UDM_VALUE_DATA_TYPE_INT,
  UDM_VALUE_DATA_TYPE_INT,
  "int",
  UdmValueIntConstructor,
  UdmValueIntDestructor,
  UdmValueIntCopy,
  UdmValueIntVarSize,
  UdmValueIntValueSize,
  UdmValueIntDataOffset,
  UdmValueIntDataSize,
  UdmValueIntDataAlignment,
  UdmValueIntDump,
  UdmValueIntFlags,
  UdmValueIntSecno,
  UdmValueIntMemUsed,
  UdmValueIntGetConstStr,
  UdmValueIntGetBool,
  UdmValueIntGetInt,
  UdmValueIntGetUInt,
  UdmValueIntGetDouble,
  UdmValueIntSetInt,
  UdmValueIntSetDouble,
  UdmValueIntSetStrn,
  UdmValueIntPrintToFile,
  UdmValueIntPrintToDSTR,
  UdmValueIntConvertCharset,
  UdmValueIntSetConv,
  UdmValueIntAppendStrn,
  UdmValueIntAppendConv,
  NULL,                    /* Methods */
};
